import unittest
import pandas as pd
import numpy as np
from os import sep
from pysight.lst_tools import Analysis
from pysight.fileIO_tools import FileIO


class TestLstTools(unittest.TestCase):
    length = 100
    df = pd.DataFrame([i for i in range(length)], columns=['abs_time'])
    analyzed = Analysis()
    dict_of_data = {
        'PMT1': pd.DataFrame([[1, 2, 3, 4, 5, 6, 7], [5, 6, 7, 8, 9, 10, 11]],
                             columns=['abs_time', 'edge', 'sweep', 'time_rel_sweep',
                                      'time_rel_pulse', 'bins_x', 'bins_y'], dtype=np.uint64),
        'Lines': pd.DataFrame([[1, 2, 3, 4], [5, 6, 7, 8]],
                              columns=['abs_time', 'edge', 'sweep', 'time_rel_sweep']),
        'Laser': pd.DataFrame([[0, 2, 3, 2], [3, 6, 7, 6]],
                              columns=['abs_time', 'edge', 'sweep', 'time_rel_sweep'])
    }
    dict_of_data['PMT1'].time_rel_pulse = dict_of_data['PMT1'].time_rel_pulse.astype(np.uint8)
    cur_file = FileIO(filename='tests_data' + sep + 'data_for_tests.lst')

    def test_laser_interpolation(self):
        pass


if __name__ == '__main__':
    unittest.main()



"""
__author__ = Hagai Hargil
"""
from typing import List
import numpy as np



def generate_output_list(movie, gui) -> List:
    """
    Main function that generates outputs based on given parameters.
    :return List containing outputs.
    """

    output_list: List = []



    return output_list


def gen_tiff(movie, gui) -> int:
    """
    Algorithm's output will be a tiff file.
    """
    movie.create_tif()
    print('Tiff stack created with name {}.tif, \none for each channel.'.format(gui.filename.get()[:-4]))
    return 1


def gen_array(movie, gui) -> List:
    """
    Algorithm's output will be a list, each cell containing the full array of data of that channel
    """
    chan_data_list = movie.create_array()
    print('Data array created.')
    return chan_data_list


def gen_single(movie, gui) -> List[np.ndarray]:
    """
    Algorithm's output will be list, with a single stack for each channel.
    """
    chan_data_list: List[np.ndarray] = movie.create_single_volume('all')
    print(r'A summed array of all volumes\frames was created.')
    return chan_data_list




"""
__author__ = Hagai Hargil
"""
import unittest
from os import sep
from pysight.fileIO_tools import FileIO


class TestFileIOTools(unittest.TestCase):
    """ Tests for new multiscaler readout functions """

    list_of_file_names = ['tests_data' + sep + 'data_for_tests.lst']
    file_io_objects = []
    for file in list_of_file_names:
        cur_obj = FileIO(file, debug=False, input_start='Frames',
                         input_stop1='PMT1', input_stop2='Lines')
        cur_obj.run()
        file_io_objects.append(cur_obj)

    def test_determine_binary(self):
        list_of_file_binary_formats = [False]
        list_of_returned_binaries = []
        for oname in self.file_io_objects:
            list_of_returned_binaries.append(oname.is_binary)

        self.assertEqual(list_of_file_binary_formats, list_of_returned_binaries)

    def test_check_range_extraction(self):
        list_of_real_range = [80000000]
        list_of_returned_range = []
        for oname in self.file_io_objects:
            metadata = oname._FileIO__get_metadata()
            list_of_returned_range.append(oname.data_range)

        self.assertEqual(list_of_real_range, list_of_returned_range)

    def test_check_time_patch_extraction(self):
        list_of_real_time_patch = ['32']
        list_of_returned_time_patch = []
        for oname in self.file_io_objects:
            metadata = oname._FileIO__get_metadata()
            list_of_returned_time_patch.append(oname.timepatch)

        self.assertEqual(list_of_real_time_patch, list_of_returned_time_patch)

    def test_check_start_of_data_value(self):
        list_of_real_start_loc = [1749]
        list_of_returned_locs = []
        for oname in self.file_io_objects:
            list_of_returned_locs.append(oname.start_of_data_pos)

        self.assertEqual(list_of_real_start_loc, list_of_returned_locs)

    def test_find_active_channels(self):
        real_list_of_active_channels = [[True, True, True, False, False, False]]
        returned_list_of_active_channels = []
        for oname in self.file_io_objects:
            metadata = oname._FileIO__get_metadata()
            returned_list_of_active_channels.append(oname.list_of_recorded_data_channels)

        self.assertEqual(returned_list_of_active_channels, real_list_of_active_channels)

    def test_create_inputs_dict(self):
        real_list_of_real_inputs_dict = [
            {'Frames': '110',
             'PMT1': '001',
             'Lines': '010'}
        ]
        list_of_returned_inputs_dict = []
        for oname in self.file_io_objects:
            list_of_returned_inputs_dict.append(oname.dict_of_input_channels)

        self.assertEqual(list_of_returned_inputs_dict, real_list_of_real_inputs_dict)

    def test_num_of_items(self):
        real_num_of_items = [-1]
        returned_num_of_items = []
        for oname in self.file_io_objects:
            returned_num_of_items.append(oname.determine_num_of_items())

        self.assertEqual(returned_num_of_items, real_num_of_items)



"""
__author__ = Hagai Hargil
"""

from unittest import TestCase
from pysight.validation_tools import *


class TestFrame(TestCase):
    """
    Tests for the validation functions
    """
    import pandas as pd
    import numpy as np

    def test_last_event_empty_dict(self):
        dict_of_data = dict()
        lines_per_frame = 1
        with self.assertRaises(ValueError):
            calc_last_event_time(dict_of_data, lines_per_frame)

    def test_last_event_wrong_lines(self):
        dict_of_data = {'PMT1': pd.DataFrame([1, 2, 3], columns=['abs_time'])}
        lines_per_frame = 0
        with self.assertRaises(ValueError):
            calc_last_event_time(dict_of_data, lines_per_frame)

    def test_last_event_only_pmt(self):
        dict_of_data = {'PMT1': pd.DataFrame([1, 5, 3], columns=['abs_time'])}
        lines_per_frame = 1
        self.assertEqual(calc_last_event_time(dict_of_data, lines_per_frame),
                         5)

    def test_last_event_with_frames(self):
        frame_data = pd.DataFrame([0, 100], columns=['abs_time'])
        dict_of_data = {'PMT1': pd.DataFrame([1, 2, 3], columns=['abs_time']),
                        'Frames': frame_data,
                        'Lines': [1, 2, 3]}
        lines_per_frame = 1
        self.assertEqual(calc_last_event_time(dict_of_data, lines_per_frame),
                         200)

    def test_last_event_with_single_frame(self):
        frame_data = pd.DataFrame([100], columns=['abs_time'])
        dict_of_data = {'PMT1': pd.DataFrame([1, 2, 3], columns=['abs_time']),
                        'Frames': frame_data,
                        'Lines': [1, 2, 3]}
        lines_per_frame = 1
        self.assertEqual(calc_last_event_time(dict_of_data, lines_per_frame),
                         200)

    def test_last_event_with_lines_less_than_needed_single_frame(self):
        line_data = pd.DataFrame([0, 10, 20], columns=['abs_time'])
        lines_per_frame = 5
        dict_of_data = {'PMT1': pd.DataFrame([1, 2, 3], columns=['abs_time']),
                        'Lines': line_data}
        self.assertEqual(calc_last_event_time(dict_of_data, lines_per_frame),
                         50)

    def test_last_event_with_lines_less_than_needed_more_frames(self):
        line_data = pd.DataFrame([0, 10, 20, 30, 40], columns=['abs_time'])
        lines_per_frame = 3
        dict_of_data = {'PMT1': pd.DataFrame([1, 2, 3], columns=['abs_time']),
                        'Lines': line_data}
        self.assertEqual(calc_last_event_time(dict_of_data, lines_per_frame),
                         60)

    def test_last_event_with_lines_just_like_needed(self):
        line_data = pd.DataFrame([0, 10, 20], columns=['abs_time'])
        lines_per_frame = 3
        dict_of_data = {'PMT1': pd.DataFrame([1, 2, 3], columns=['abs_time']),
                        'Lines': line_data}
        self.assertEqual(calc_last_event_time(dict_of_data, lines_per_frame),
                         30)

    def test_last_event_with_lines_more_than_needed(self):
        line_data = pd.DataFrame(np.arange(0, 100, 10), columns=['abs_time'])
        lines_per_frame = 3
        dict_of_data = {'PMT1': pd.DataFrame([1, 2, 3], columns=['abs_time']),
                        'Lines': line_data}
        self.assertEqual(calc_last_event_time(dict_of_data, lines_per_frame),
                         120)



"""
#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = Hagai Hargil
"""
import attr
from attr.validators import instance_of
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from typing import List, Iterator, Tuple, Iterable, Generator, Deque
from numba import jit, float64, uint64, int64
from collections import OrderedDict, namedtuple, deque
import warnings


@attr.s
class Movie(object):
    """
    A holder for Volume objects to be displayed consecutively.
    """
    data            = attr.ib()
    reprate         = attr.ib(default=80e6, validator=instance_of(float))
    x_pixels        = attr.ib(default=512, validator=instance_of(int))
    y_pixels        = attr.ib(default=512, validator=instance_of(int))
    z_pixels        = attr.ib(default=1, validator=instance_of(int))
    name            = attr.ib(default='Movie', validator=instance_of(str))
    binwidth        = attr.ib(default=800e-12, validator=instance_of(float))
    fill_frac       = attr.ib(default=80.0, validator=instance_of(float))
    big_tiff        = attr.ib(default=True, validator=instance_of(bool))
    bidir           = attr.ib(default=True, validator=instance_of(bool))
    num_of_channels = attr.ib(default=1, validator=instance_of(int))
    outputs         = attr.ib(default={}, validator=instance_of(dict))
    censor          = attr.ib(default=False, validator=instance_of(bool))
    flim            = attr.ib(default=False, validator=instance_of(bool))
    lst_metadata    = attr.ib(default={}, validator=instance_of(dict))
    exp_params      = attr.ib(default={}, validator=instance_of(dict))
    summed          = attr.ib(init=False)
    stack           = attr.ib(init=False)

    @property
    def bins_bet_pulses(self) -> int:
        if self.flim:
            return int(np.ceil(1 / (self.reprate * self.binwidth)))
        else:
            return 1

    @property
    def list_of_volume_times(self) -> List[np.uint64]:
        """ All volumes start-times in the movie. """

        volume_times = np.unique(self.data.index.get_level_values('Frames')).astype(np.uint64)

        if len(volume_times) > 1:
            diff_between_frames = np.mean(np.diff(volume_times))
        else:
            diff_between_frames = np.uint64(np.max(self.data['time_rel_frames']))

        volume_times = list(volume_times)
        volume_times.append(np.uint64(volume_times[-1] + diff_between_frames))

        return volume_times

    def run(self):
        """
        Main pipeline for the movie object
        :return:
        """
        self.__create_outputs()
        self.__print_outputs()
        print("Movie object created, analysis done.")

    def gen_of_volumes(self, channel_num: int) -> Iterator:
        """
        Populate the deque containing the volumes as a generator.
        Creates a list for each channel in the data. Channels start with 1.
        """

        list_of_frames: List[int] = self.list_of_volume_times  # saves a bit of computation
        for idx, current_time in enumerate(list_of_frames[:-1]):  # populate deque with frames
            cur_data = self.data.xs(key=(current_time, channel_num), level=('Frames', 'Channel'), drop_level=False)
            if not cur_data.empty:
                yield Volume(data=cur_data, x_pixels=self.x_pixels, y_pixels=self.y_pixels,
                             z_pixels=self.z_pixels, number=idx, abs_start_time=current_time,
                             reprate=self.reprate, binwidth=self.binwidth, empty=False,
                             end_time=(list_of_frames[idx + 1] - list_of_frames[idx]),
                             bidir=self.bidir, fill_frac=self.fill_frac, censor=self.censor)
            else:
                yield Volume(data=cur_data, x_pixels=self.x_pixels,
                             y_pixels=self.y_pixels, z_pixels=self.z_pixels, number=idx,
                             reprate=self.reprate, binwidth=self.binwidth, empty=True,
                             end_time=(list_of_frames[idx + 1] - list_of_frames[idx]),
                             bidir=self.bidir, fill_frac=self.fill_frac, censor=self.censor)

    def __create_outputs(self) -> None:
        """
        Create the outputs according to the outputs dictionary.
        """
        if not self.outputs:
            warnings.warn("No outputs requested. Data is still accessible using the dataframe variable.")
            return

        self.summed = {i: 0 for i in range(1, self.num_of_channels + 1)}
        self.stack = {}
        VolTuple = namedtuple('VolumeHist', ('hist', 'edges'))
        data_of_vol = VolTuple

        for chan in range(1, self.num_of_channels + 1):
            if 'tif' in self.outputs:
                deque_of_vols = self.__create_tif(channel=chan)
            else:
                deque_of_vols = deque()
                for vol in self.gen_of_volumes(channel_num=chan):
                    data_of_vol.hist, data_of_vol.edges = vol.create_hist()
                    # Censor correction
                    if self.censor:
                        data_of_vol.hist = self.__nano_flim(data=data_of_vol.hist)

                    deque_of_vols.append(data_of_vol.hist)
                    self.summed[chan] += data_of_vol.hist

            assert len(deque_of_vols) == len(self.list_of_volume_times) - 1
            self.stack[chan] = deque_of_vols

    def __create_tif(self, channel) -> Deque[np.ndarray]:
        """ Create all volumes, one-by-one, and save them as tiff. """

        from tifffile import TiffWriter

        deque_of_vols = deque()
        with TiffWriter(f'{self.name[:-4]}_Chan_{channel}.tif', bigtiff=self.big_tiff,
                        software='PySight', imagej=True) as tif:
            for vol in self.gen_of_volumes(channel_num=channel):
                hist, edges = vol.create_hist()
                self.summed[channel] += hist
                deque_of_vols.append(hist)
                try:
                    # Dimension order: Z-Lifetime (C)-T-Y-X-S
                    # Might need to swap Z and T.
                    tif.save(hist.reshape(self.z_pixels, self.bins_bet_pulses,
                                          1, self.y_pixels, self.x_pixels, 1),
                             metadata=self.lst_metadata)
                except PermissionError:
                    warnings.warn("Permission Error: Not allowed to save file to original directory.")

        print(f"Finished writing `.tif` of channel {channel}.")
        return deque_of_vols

    def __print_outputs(self) -> None:
        """
        Print to console the outputs that were generated.
        """
        if not self.outputs:
            return

        print('======================================================= \nOutputs:\n--------')
        if 'tif' in self.outputs:
            print(f'Tiff stack created with name {self.name[:-4]}.tif, \none for each channel.')

        if 'full' in self.outputs:
            print('The full data is present in dictionary form (key per channel) under `movie.stack`')

        if 'summed' in self.outputs:
            print('Summed data is present in dictionary form (key per channel) under `movie.summed`.')

    def __nano_flim(self, data: np.ndarray) -> None:
        pass


@attr.s(slots=True)
class Volume(object):
    """
    A Movie() is a sequence of volumes. Each volume contains frames in a plane.
    """
    data           = attr.ib(validator=instance_of(pd.DataFrame))
    x_pixels       = attr.ib(default=512, validator=instance_of(int))
    y_pixels       = attr.ib(default=512, validator=instance_of(int))
    z_pixels       = attr.ib(default=1, validator=instance_of(int))
    number         = attr.ib(default=1, validator=instance_of(int))  # the volume's ordinal number
    reprate        = attr.ib(default=80e6, validator=instance_of(float))  # laser repetition rate, relevant for FLIM
    end_time       = attr.ib(default=np.uint64(100), validator=instance_of(np.uint64))
    binwidth       = attr.ib(default=800e-12, validator=instance_of(float))
    bidir          = attr.ib(default=False, validator=instance_of(bool))  # Bi-directional scanning
    fill_frac      = attr.ib(default=80.0, validator=instance_of(float))
    abs_start_time = attr.ib(default=np.uint64(0), validator=instance_of(np.uint64))
    empty          = attr.ib(default=False, validator=instance_of(bool))
    censor         = attr.ib(default=False, validator=instance_of(bool))

    @property
    def metadata(self) -> OrderedDict:
        """
        Creates the metadata of the volume to be created, to be used for creating the actual images
        using histograms. Metadata can include the first photon arrival time, start and end of volume, etc.
        :return: Dictionary of all needed metadata.
        """

        metadata = OrderedDict()
        jitter = 0.02  # 2% of jitter of the signals that creates volumes

        # Volume metadata
        volume_start: int = 0
        metadata['Volume'] = Struct(start=volume_start, end=self.end_time, num=self.x_pixels + 1)

        # y-axis metadata
        y_start, y_end, self.empty = metadata_ydata(data=self.data, jitter=jitter, bidir=self.bidir,
                                                    fill_frac=self.fill_frac)
        if y_end == 1:  # single pixel in frame
            metadata['Y'] = Struct(start=y_start, end=self.end_time, num=self.y_pixels + 1)
        else:
            metadata['Y'] = Struct(start=y_start, end=y_end, num=self.y_pixels + 1)

        # z-axis metadata
        if 'Phase' in self.data.columns:
            z_start = -1
            z_end = 1
            metadata['Z'] = Struct(start=z_start, end=z_end, num=self.z_pixels + 1)

        # Laser pulses metadata
        if 'time_rel_pulse' in self.data.columns:
            try:
                laser_start = 0
                laser_end = np.ceil(1 / (self.reprate * self.binwidth)).astype(np.uint8)
                metadata['Laser'] = Struct(start=laser_start, end=laser_end, num=laser_end + 1)
            except ZeroDivisionError:
                laser_start = 0
                warnings.warn('No laser reprate provided. Assuming 80.3 MHz.')
                laser_end = np.ceil(1 / (80.3e6 * self.binwidth)).astype(np.uint8)
                metadata['Laser'] = Struct(start=laser_start, end=laser_end, num=laser_end + 1)

        return metadata

    def __create_hist_edges(self):
        """
        Create three vectors that will create the grid of the frame. Uses Numba internal function for optimization.
        :return: Tuple of np.array
        """
        metadata = self.metadata
        list_of_edges = []

        if self.empty is not True:
            for num_of_dims, key in enumerate(metadata, 1):
                if 'Volume' == key:
                    list_of_edges.append(self.__create_line_array())
                else:
                    list_of_edges.append(create_linspace(start=metadata[key].start,
                                                         stop=metadata[key].end,
                                                         num=metadata[key].num))

            return list_of_edges, num_of_dims
        else:
            return list(np.ones(len(metadata)))

    def __create_line_array(self):
        """
        Generates the edges of the final histogram using the line signal from the data
        :return: np.array
        """
        lines = self.data.index.get_level_values('Lines').categories.values
        lines.sort()
        if len(lines) > 1:
            if len(lines) < self.x_pixels:
                raise ValueError(f'Not enough line events in volume number {self.number}.')
            else:
                mean_diff = np.diff(lines).mean()
                return np.r_[lines[:self.x_pixels], np.array([lines[self.x_pixels - 1] + mean_diff], dtype='uint64')]
        else:  # single pixel frames, perhaps
            return np.r_[lines, lines + self.end_time]

    def create_hist(self) -> Tuple[np.ndarray, Iterable]:
        """
        Create the histogram of data using calculated edges.
        :return: np.ndarray of shape [num_of_cols, num_of_rows] with the histogram data, and edges
        """

        list_of_data_columns = []

        if not self.empty:
            list_of_edges, num_of_dims = self.__create_hist_edges()
            list_of_data_columns.append(self.data['time_rel_frames'].values)
            list_of_data_columns.append(self.data['time_rel_line'].values)
            try:
                list_of_data_columns.append(self.data['Phase'].values)
            except KeyError:
                pass
            try:
                list_of_data_columns.append(self.data['time_rel_pulse'].values)
            except KeyError:
                pass

            data_to_be_hist = np.reshape(list_of_data_columns, (num_of_dims, self.data.shape[0])).T

            assert data_to_be_hist.shape[0] == self.data.shape[0]
            assert len(list_of_data_columns) == data_to_be_hist.shape[1]

            hist, edges = np.histogramdd(sample=data_to_be_hist, bins=list_of_edges)

            if self.censor:
                hist = self.__censor_correction(hist)

            return hist.astype(np.int16), edges
        else:
            return np.zeros((self.x_pixels, self.y_pixels, self.z_pixels), dtype=np.int16), (0, 0, 0)

    def show(self) -> None:
        """ Show the Volume. Mainly for debugging purposes, as the Movie object doesn't use it. """

        hist, edges = self.create_hist()
        plt.figure()
        # plt.imshow(hist, cmap='gray')
        plt.title(f'Volume number {self.number}')
        plt.axis('off')

    def __censor_correction(self, data) -> np.ndarray:
        """
        Add censor correction to the data after being histogrammed
        :param data:
        :return:
        """
        return data


def validate_number_larger_than_zero(instance, attribute, value: int=0):
    """
    Validator for attrs module - makes sure line numbers and row numbers are larger than 0.
    """

    if value >= instance.attribute:
        raise ValueError(f"{attribute} has to be larger than 0.")


@jit((float64[:](int64, uint64, uint64)), nopython=True, cache=True)
def create_linspace(start, stop, num):
    linspaces = np.linspace(start, stop, num)
    assert np.all(np.diff(linspaces) > 0)
    return linspaces


def metadata_ydata(data: pd.DataFrame, jitter: float=0.02, bidir: bool = True, fill_frac: float = 0):
    """
    Create the metadata for the y-axis.
    """
    lines_start: int = 0
    empty: bool = False

    unique_indices: np.ndarray = np.unique(data.index.get_level_values('Lines'))
    if unique_indices.shape[0] > 1:  # TODO: Doesn't make sense that there's a single line in a frame
        diffs: np.ndarray = np.diff(unique_indices)
        diffs_max: float = diffs.max()
    else:
        empty = False
        lines_end = 1
        return lines_start, lines_end, empty

    try:
        if diffs_max > ((1 + 4 * jitter) * np.mean(diffs)):  # Noisy data
            lines_end = np.mean(diffs) * (1 + jitter)
        else:
            lines_end = int(diffs_max)
    except ValueError:
        lines_end = 0
        empty = True

    # Case where it's a unidirectional scan and we dump back-phase photons
    if not bidir:
        lines_end /= 2

    if fill_frac > 0:
        lines_end = lines_end * fill_frac/100

    return lines_start, int(lines_end), empty


@attr.s
class Struct(object):
    """ Basic struct-like object for data keeping. """

    start = attr.ib()
    end = attr.ib()
    num = attr.ib(default=None)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import io
import os
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import relpath
from os.path import splitext

from setuptools import Extension
from setuptools import find_packages
from setuptools import setup
from setuptools.command.build_ext import build_ext

try:
    # Allow installing package without any Cython available. This
    # assumes you are going to include the .c files in your sdist.
    import Cython
except ImportError:
    Cython = None

def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()

class CustomBuildExtCommand(build_ext):
    """build_ext command for use when numpy headers are needed."""
    def run(self):

        # Import numpy here, only when headers are needed
        import numpy

        # Add numpy headers to include_dirs
        self.include_dirs.append(numpy.get_include())

        # Call original build_ext command
        build_ext.run(self)


# Enable code coverage for C code: we can't use CFLAGS=-coverage in tox.ini, since that may mess with compiling
# dependencies (e.g. numpy). Therefore we set SETUPPY_CFLAGS=-coverage in tox.ini and copy it to CFLAGS here (after
# deps have been safely installed).
if 'TOXENV' in os.environ and 'SETUPPY_CFLAGS' in os.environ:
    os.environ['CFLAGS'] = os.environ['SETUPPY_CFLAGS']

setup(
    name='pysight',
    version='0.5.18',
    license='BSD',
    description='Create images from photon lists generated by a multiscaler.',
    long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.rst')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
    ),
    author='Hagai Hargil',
    author_email='hagaihargil@gmail.com',
    url='https://github.com/HagaiHargil/python-pysight',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Utilities',
    ],
    keywords=[
        'multiscaler', 'photon counting'
    ],
    cmdclass = {'build_ext': CustomBuildExtCommand},
    install_requires=[
        'numpy >= 1.13',
        'matplotlib >= 2.0',
        'pandas >= 0.20',
        'attrs >= 17.0',
        'cython >= 0.24',
        'tables',
        'scipy',
        'scikit-learn'
    ],
    extras_require={
    },
    setup_requires=[
        'cython',
    ] if Cython else [],
    ext_modules=[
        Extension(
            splitext(relpath(path, 'src').replace(os.sep, '.'))[0],
            sources=[path],
            include_dirs=[dirname(path)]
        )
        for root, _, _ in os.walk('src')
        for path in glob(join(root, '*.pyx' if Cython else '*.c'))
    ],
    data_files=['src' + os.sep + 'pysight' + os.sep + 'configs' + os.sep + 'default.json']
)



"""
__author__ = Hagai Hargil
"""
import pstats, cProfile
from pysight.main_multiscaler_readout import main_data_readout
from pysight.tkinter_gui_multiscaler import GUIApp
from pysight.tkinter_gui_multiscaler import verify_gui_input

import pyximport
pyximport.install()

gui = GUIApp()
gui.root.mainloop()
verify_gui_input(gui)
cProfile.runctx("main_data_readout(gui)", globals(), locals(), "Profile.prof")

s = pstats.Stats("Profile.prof")
s.strip_dirs().sort_stats("time").print_stats()



"""
__author__ = Hagai Hargil
"""


import unittest
from pysight.censor_tools import CensorCorrection
from pysight.censor_tools import CensoredVolume
from pysight.movie_tools import Movie, Volume
import pandas as pd
from collections import deque, namedtuple
import numpy as np


class TestCensorTools(unittest.TestCase):
    length = 100
    df = pd.DataFrame([i for i in range(length)], columns=['abs_time'])
    tuptype = namedtuple('TestCensor', ('hist', 'edges'))
    cens = tuptype
    cens.hist = np.array([i for i in range(length)])
    cens.edges = np.array([1, 4, 8, 18, 25, length - 1])

    dict_of_data = {
        'PMT1': pd.DataFrame([[1, 2, 3, 4, 5, 6, 7], [5, 6, 7, 8, 9, 10, 11]],
                             columns=['abs_time', 'edge', 'sweep', 'time_rel_sweep',
                                      'time_rel_pulse', 'bins_x', 'bins_y'], dtype=np.uint64),
        'Lines': pd.DataFrame([[1, 2, 3, 4], [5, 6, 7, 8]],
                             columns=['abs_time', 'edge', 'sweep', 'time_rel_sweep']),
        'Laser': pd.DataFrame([[0, 2, 3, 2], [3, 6, 7, 6]],
                             columns=['abs_time', 'edge', 'sweep', 'time_rel_sweep'])
    }
    dict_of_data['PMT1'].time_rel_pulse = dict_of_data['PMT1'].time_rel_pulse.astype(np.uint8)
    movie = Movie(data=dict_of_data['PMT1'])

    def test_allocate_some_photons(self):
        censored = CensoredVolume(df=self.df, vol=Volume(data=self.dict_of_data['PMT1']),
                                  offset=0)
        photons = self.dict_of_data['PMT1']
        photons.set_index(keys=['bins_x', 'bins_y'], drop=True, inplace=True)
        idx_list = np.array([7])
        result = np.array([0], dtype=object)
        res_hist = np.histogram(np.array([5], dtype=object),
                                            bins=np.arange(0, censored.bins_bet_pulses+1, dtype=np.uint8))[0]
        result[0] = res_hist
        returned = censored._CensoredVolume__allocate_photons_to_bins(idx_list, photons)
        self.assertTrue(np.all(result[0] == returned[0]))

    def test_append_laser(self):
        censored = CensorCorrection(raw=self.dict_of_data,
                                    movie=self.movie,
                                    all_laser_pulses=self.dict_of_data['Laser'])

        censored.append_laser_line()
        res = pd.DataFrame([[0, 2, 3, 2], [3, 6, 7, 6], [19, 0, 7, 22]],
                           columns=['abs_time', 'edge', 'sweep', 'time_rel_sweep'])
        self.assertTrue(censored.raw['Laser'].equals(res))
